<?php

declare(strict_types=1);

namespace Drupal\devel_oidc\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\openid_connect\OpenIDConnectSession;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Mock OIDC login form.
 */
final class OidcMockLoginForm extends FormBase {

  /**
   * OidcMockLoginForm constructor.
   */
  public function __construct(
    private readonly OpenIDConnectSession $session,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): OidcMockLoginForm {
    return new static(
      $container->get('openid_connect.session')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'devel_oidc_mock_login_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [];

    $form['email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email'),
    ];

    $form['groups'] = [
      '#type' => 'textfield',
      '#title' => $this->t('AAD Groups'),
      '#description' => $this->t('Group names, delimited by semicolons.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Login'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    if (!filter_var($form_state->getValue('email'), FILTER_VALIDATE_EMAIL)) {
      $form_state->setError($form['email'], $this->t('Invalid email address'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $routeParameters = [
      'openid_connect_client' => $form_state->getBuildInfo()['args'][0],
    ];

    $groups = explode(';', $form_state->getValue('groups'));
    $token = base64_encode(Json::encode([
      'sub' => $form_state->getValue('email'),
      'groups' => $groups,
      'aad_groups' => $groups,
    ]));

    $options = [
      'query' => [
        'state' => $this->session->retrieveStateToken(FALSE),
        'code' => $token,
      ],
    ];

    $url = Url::fromRoute('openid_connect.redirect_controller_redirect', $routeParameters, $options);
    $form_state->setRedirectUrl($url);
  }

}
